﻿// Learn more about F# at http://fsharp.net
// See the 'F# Tutorial' project for more help.

namespace Samples.FSharp.DesignTimeSeparation

open System.Reflection
open Microsoft.FSharp.Core.CompilerServices
open Samples.FSharp.ProvidedTypes
open Microsoft.FSharp.Quotations

[<TypeProvider>]
type public TypeProvider1(config: TypeProviderConfig) as this =
    inherit TypeProviderForNamespaces()
    let ass = config.RuntimeAssembly
    let runtimeAssembly = System.Reflection.Assembly.ReflectionOnlyLoadFrom(ass)     
    
    let objectBase = runtimeAssembly.GetType("Samples.MiniTypeSpace.Runtime.ObjectBase")
    let runtimeMethods = runtimeAssembly.GetType("Samples.MiniTypeSpace.Runtime.RuntimeMethods")    
    let createMethod = runtimeMethods.GetMethod("Create")
    let getContentsMethod = runtimeMethods.GetMethod("GetContents")
    
    // Get the assembly and namespace used to house the provided types
    let rootNamespace = "Samples.ShareInfo.TPTest"
    let baseTy = objectBase

    let newT = ProvidedTypeDefinition(runtimeAssembly, rootNamespace, "TPTestType", Some baseTy)
    
    let ctor = ProvidedConstructor([ProvidedParameter("Contents",typeof<string>)],
                                       InvokeCode = (fun args -> Expr.Call(createMethod, [ Expr.Value("中国") ])))

    // This is property. At runtime, the property just gets the string value
    let prop2 = ProvidedProperty("属性", typeof<string>, 
                                         GetterCode= (fun args -> Expr.Call(getContentsMethod, [ args.[0] ])))
   

    do prop2.AddXmlDocDelayed(fun () -> "xml comment")    
    do 
        newT.AddMember(prop2)
        newT.AddMember(ctor)
    
    do this.AddNamespace(rootNamespace, [newT])

[<TypeProviderAssembly>]
do ()