﻿module Code2_50

// define an IPoint2D interface
// need the and keyword to resolve the type-resolution problem
type IPoint2D = 
    inherit I2DLocation

    // this function takes I2DLocation as a parameter, which is defined below
    abstract member GetDistance : I2DLocation -> float

// define a 2D location interface
and I2DLocation = interface 
    abstract member X : float with get, set
    abstract member Y : float with get, set
end 



// define a Point2D class with a parameterized primary constructor
type Point2D(xValue:double, yValue:double) as this =
    // define field x
    let mutable x = xValue

    // define a static field named count
    static let mutable count = 0

    // perform a do binding to increase the count
    do
        count <- count + 1


    //additional constructor
    new() = Point2D(0.,0.)

    // print out message
    member this.Print() = printfn "In class Point2D"
 
    // define an X property
    member this.X with get() = x
                  and set(v) = x <- v

    // define a Y property using an auto-implemented property 
    member val Y = yValue with get, set

    // define a function to get the distance from the current point
    abstract GetDistance : point: I2DLocation -> double
    default this.GetDistance(point) = 
        let xDif = this.X - point.X
        let yDif = this.Y - point.Y
        let distance = sqrt (xDif**2. + yDif**2.)
        distance

    // define a static method to return a Point2D from (x,y) tuple
    static member FromXY (x:double , y:double ) = 
        Point2D(x,y)

    interface I2DLocation with
        member this.X with get() = this.X and set(v) = this.X <- v
        member this.Y with get() = this.Y and set(v) = this.Y <- v
    interface IPoint2D with
        member this.GetDistance(p) = this.GetDistance(p)

    override this.ToString() = sprintf "Point2D(%f, %f)" this.X this.Y
