﻿module Code7_44

type Location = { X : int; Y : int }

[<System.Diagnostics.DebuggerDisplay("({Location.X},{Location.Y})")>]
type Node() as this = 
    member val IsBlock = false with get, set
    member val PreviousNode = this with get, set
    member val Location = {X = 0; Y = 0} with get, set
    member val H = 0 with get, set
    member val MoveCost = 1000 with get, set
    member this.TotalCost = this.MoveCost + this.H

type Path = Location list

let graph = [
        [1;1;1;1;1;1;1]
        [1;1;1;0;1;1;1]
        [1;1;1;0;1;1;1]
        [1;1;1;0;1;1;1]
        [1;1;1;0;1;1;1]
        [1;1;1;1;1;1;1]
        [1;1;1;1;1;1;1]
            ]

let nodeGraph = 
    graph
    |> List.mapi (fun y row ->
                    row 
                    |> List.mapi (fun x n -> 
                        let loc = {X = x; Y = y}
                        Node(IsBlock= (if n = 1 then false else true),
                             Location = loc))
                )

let rightMax = 6

let getNeighbors (point:Location) = 
    let getLeft (point:Location) = 
        if point.X = 0 then None
        else Some { X = point.X - 1; Y = point.Y }
    let getTop (point:Location) = 
        if point.Y = 0 then None
        else Some { X = point.X; Y = point.Y - 1 }
    let getRight (point:Location) = 
        if point.X = rightMax then None
        else Some { X = point.X + 1; Y = point.Y }
    let getBottom (point:Location) = 
        if point.Y=rightMax then None
        else Some { X = point.X; Y = point.Y + 1 }
    let getLeftTop (point:Location) = 
        if point.X = 0 || point.Y = 0 then None
        else Some { X = point.X - 1; Y= point.Y - 1 }
    let getRightTop (point:Location) = 
        if point.X = rightMax || point.Y = 0 then None
        else Some { X = point.X + 1; Y=point.Y - 1 }
    let getRightBottom (point:Location) = 
        if point.X = rightMax || point.Y = rightMax then None
        else Some { X = point.X + 1; Y = point.Y + 1 }
    let getLeftBottom (point:Location) = 
        if point.X = 0 || point.Y = rightMax then None
        else Some { X = point.X - 1; Y=point.Y + 1 }
    let result = 
        [(10, getLeft point); (10, getTop point); (10, getRight point); (10, getBottom point);
         (14, getLeftTop point); (14, getRightTop point); (14, getRightBottom point); (14, 
getLeftBottom point);
        ]
    let result = 
        result
        |> List.filter (fun (cost, n) -> match n with Some _ -> true | None -> false )
        |> List.map (fun (cost, (Some n)) -> cost,n)
        |> List.filter (fun (cost, n) -> nodeGraph.[n.Y].[n.X].IsBlock |> not)
        |> List.map (fun (cost, n) -> cost, nodeGraph.[n.Y].[n.X])
    result

let mutable openList : Node list = []
let mutable closedList : Node list = []

let addOpenList node =
    openList <- openList @ [node]

let addClosedList node = 
    closedList <- closedList @ [node]

let closedListContain (endPoint:Node) = 
    closedList
    |> Seq.exists (fun n -> n.Location = endPoint.Location)

let openListContain (node:Node) =
    openList 
    |> Seq.exists (fun n -> n.Location = node.Location)

let ``a*`` start (endPoint:Node) = 
    addOpenList start
    nodeGraph
    |> List.iter (fun list -> 
                    list
                    |> List.iter (fun n ->
                                    n.H <- 10 * (abs (n.Location.X - endPoint.Location.X) 
                                           + abs (n.Location.Y - endPoint.Location.Y))))
    while not <| closedListContain endPoint && openList.Length > 0 do
        let currentNode::tail = openList |> List.sortBy (fun n -> n.TotalCost)
        openList <- tail
        addClosedList currentNode

        let neighbours:(int*Node) list = 
                getNeighbors currentNode.Location
                |> List.filter (fun (_, node) -> node |> closedListContain |> not)
        for cost, node in neighbours do
            if node.MoveCost > currentNode.MoveCost + cost then
                node.PreviousNode <- currentNode
                node.MoveCost <- currentNode.MoveCost + cost
            if not <| openListContain node then
                addOpenList node

let start = nodeGraph.[0].[0]
start.MoveCost <- 0
let endPoint = nodeGraph.[2].[5]

``a*`` start endPoint
