﻿namespace WorkerNode

open System
open System.Collections.Generic
open System.Diagnostics
open System.Linq
open System.Net
open System.Threading
open Microsoft.WindowsAzure
open Microsoft.WindowsAzure.Diagnostics
open Microsoft.WindowsAzure.ServiceRuntime
open Microsoft.WindowsAzure.StorageClient

type WorkerRole() =
    inherit RoleEntryPoint() 

    // This is a sample worker implementation. Replace with your logic.

    let log message kind = Trace.WriteLine(message, kind)
    let random = Random()

    override wr.Run() =

        let connectionStringName = "QueueConnectionString"
        let storageAccount = CloudStorageAccount.Parse(CloudConfigurationManager.GetSetting(connectionStringName))
        let queueClient = storageAccount.CreateCloudQueueClient()
        let queue = queueClient.GetQueueReference("myqueue")
        let queue2 = queueClient.GetQueueReference("resultqueue")

        ignore <| queue.CreateIfNotExist()
        ignore <| queue2.CreateIfNotExist()

        let mutable msg = queue.GetMessage()
        while msg.DequeueCount<>1 do
            Thread.Sleep(random.Next(5, 100))
            msg <- queue.GetMessage()
        queue.DeleteMessage(msg)

        let data = 
            msg.AsString.Split([| "," |], StringSplitOptions.RemoveEmptyEntries)
            |> Array.map Convert.ToInt32

        let sum = data |> Seq.sum

        queue2.AddMessage(CloudQueueMessage(sum.ToString()))

        log "WorkerNode entry point called" "Information"
        while(true) do 
            Thread.Sleep(10000)
            log "Working" "Information"

    override wr.OnStart() = 

        // Set the maximum number of concurrent connections 
        ServicePointManager.DefaultConnectionLimit <- 12
       
        // For information on handling configuration changes
        // see the MSDN topic at http://go.microsoft.com/fwlink/?LinkId=166357.

        base.OnStart()
