﻿// Learn more about F# at http://fsharp.net
// See the 'F# Tutorial' project for more help.

namespace Samples.FSharp.XMLTypeProvider

open System.Reflection
open Microsoft.FSharp.Core.CompilerServices
open Samples.FSharp.ProvidedTypes
open System.Xml.Linq
open Microsoft.FSharp.Quotations
open System

type MyProductionCode() = class
    member this.Work() = 
        let xml = XDocument.Load("XmlFile1.xml") 
        let start = xml.Descendants(XName.Get("Start")) |> Seq.head
        let end' = xml.Descendants(XName.Get("End")) |> Seq.head
        
        // if start < end, something bad will happen
        if Convert.ToInt32(start.Value) >= Convert.ToInt32(end'.Value) then 
            failwith "report illegal parameter from runtime"
end

type MyTest() = class
    member this.Test1(xml:string) = 
        let xml = XDocument.Load(xml) 
        let start = xml.Descendants(XName.Get("Start")) |> Seq.head
        let end' = xml.Descendants(XName.Get("End")) |> Seq.head
        if Convert.ToInt32(start.Value) >= Convert.ToInt32(end'.Value) then 
            failwith "report illegal parameter from design-time"
end

[<AutoOpen>]
module TestModule = 
    let test = MyTest()

[<TypeProvider>]
type public TypeProvider1(cfg : Microsoft.FSharp.Core.CompilerServices.TypeProviderConfig) as this =
    inherit TypeProviderForNamespaces()
    
    // Get the assembly and namespace used to house the provided types
    let thisAssembly = Assembly.GetExecutingAssembly()
    let rootNamespace = "Samples.ShareInfo.TPTest"
    let baseTy = typeof<obj>

    let newT = ProvidedTypeDefinition(thisAssembly, rootNamespace, "XMLTypeProvider", Some baseTy)
    
    let ctor = ProvidedConstructor(parameters = [], InvokeCode = fun args -> <@@ (* base class initialization or null*) () @@>) 

    let staticParams = [ProvidedStaticParameter("xml file name", typeof<string>)]
    do newT.DefineStaticParameters(
        parameters=staticParams, 
        instantiationFunction=(fun typeName parameterValues ->
                let baseTy = typeof<MyProductionCode>

                let ty = ProvidedTypeDefinition(
                    thisAssembly, 
                    rootNamespace, 
                    typeName, 
                    baseType = Some baseTy)

                let xmlFileName = System.IO.Path.Combine(cfg.ResolutionFolder, parameterValues.[0] :?> string)
                let m = ProvidedMethod(
                            methodName = "Work",
                            parameters = [],
                            returnType = typeof<unit>,
                            IsStaticMethod = false,
                            InvokeCode = fun args -> 
                                test.Test1(xmlFileName)                                
                                <@@ (%%args.[0]:MyProductionCode).Work() @@>                                    
                            )

                let ctor = ProvidedConstructor(
                                        parameters = [], 
                                        InvokeCode = fun args -> <@@ MyProductionCode() @@>) 

                ty.AddMember(ctor)
                ty.AddMember(m)
    
                ty.AddXmlDoc "xml comment"
                ty))

    do 
        newT.AddMember(ctor)
    
    do this.AddNamespace(rootNamespace, [newT])

[<TypeProviderAssembly>]
do ()